/* ###################################################################
**     THIS COMPONENT MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename    : TmDt1.c
**     Project     : ProcessorExpert
**     Processor   : MKL25Z128VLK4
**     Component   : GenericTimeDate
**     Version     : Component 01.001, Driver 01.00, CPU db: 3.00.000
**     Compiler    : GNU C Compiler
**     Date/Time   : 2013-07-01, 17:48, # CodeGen: 0
**     Abstract    :
**
**     Settings    :
**
**     Contents    :
**         AddTick - void TmDt1_AddTick(void);
**         SetTime - byte TmDt1_SetTime(byte Hour, byte Min, byte Sec, byte Sec100);
**         GetTime - byte TmDt1_GetTime(TIMEREC *Time);
**         SetDate - byte TmDt1_SetDate(word Year, byte Month, byte Day);
**         GetDate - byte TmDt1_GetDate(DATEREC *Date);
**         Init    - void TmDt1_Init(void);
**
**     (c) Copyright Freescale Semiconductor, 2012
**     http: www.freescale.com
**     Source code is based on the original TimeDate Processor Expert component.
** ###################################################################*/
/*!
** @file TmDt1.c
** @version 01.00
** @brief
**
*/         
/*!
**  @addtogroup TmDt1_module TmDt1 module documentation
**  @{
*/         

/* MODULE TmDt1. */

#include "TmDt1.h"

#define TmDt1_TICK_TIME_MS  10         /* timer tick time in milliseconds, set in properties */
#define TmDt1_TICKS_PER_S  (1000/TmDt1_TICK_TIME_MS) /* number of timer ticks per second */

static byte CntDay;                    /* Day counter */
static byte CntMonth;                  /* Month counter */
static word CntYear;                   /* Year Counter */
static dword TotalHthH;                /* Software tick counter (1 tick = TmDt1_TICK_TIME_MS ms) */

/* Table of month length (in days) */
static const  byte ULY[12] = {31U,28U,31U,30U,31U,30U,31U,31U,30U,31U,30U,31U}; /* Un-leap-year */
static const  byte  LY[12] = {31U,29U,31U,30U,31U,30U,31U,31U,30U,31U,30U,31U}; /* Leap-year */
/*
** ===================================================================
**     Method      :  TmDt1_SetTime (component GenericTimeDate)
**     Description :
**         This method sets a new actual time.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Hour            - Hours (0 - 23)
**         Min             - Minutes (0 - 59)
**         Sec             - Seconds (0 - 59)
**         Sec100          - Hundredth of seconds (0 - 99)
**     Returns     :
**         ---             - Error code
** ===================================================================
*/
#ifdef __HIWARE__
  #pragma MESSAGE DISABLE C5905 /* multiplication with one (happens if TmDt1_TICKS_PER_S is 100) */
#endif
byte TmDt1_SetTime(byte Hour, byte Min, byte Sec, byte Sec100)
{
  if ((Sec100 > 99U) || (Sec > 59U) || (Min > 59U) || (Hour > 23U)) { /* Test correctnes of given time */
    return ERR_RANGE;                  /* If not correct then error */
  }
  EnterCritical();
  TotalHthH =   (3600UL*TmDt1_TICKS_PER_S*(dword)Hour)
              + (60UL*TmDt1_TICKS_PER_S*(dword)Min)
              + (TmDt1_TICKS_PER_S*(dword)Sec)
              + ((TmDt1_TICKS_PER_S/100)*(dword)Sec100); /* Load given time re-calculated to TmDt1_TICK_TIME_MS ms ticks into software tick counter */
  ExitCritical();
  return ERR_OK;                       /* OK */
}
#ifdef __HIWARE__
  #pragma MESSAGE DEFAULT C5905 /* multiplication with one  */
#endif

/*
** ===================================================================
**     Method      :  TmDt1_AddTick (component GenericTimeDate)
**     Description :
**         Needs to be called periodically by the application to
**         increase the time tick count.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void TmDt1_AddTick(void)
{
  const byte * ptr;                    /* Pointer to ULY/LY table */

  TotalHthH += 0x01UL;                 /* Software timer counter increment by timer period */
  if (TotalHthH >= 24*3600UL*TmDt1_TICKS_PER_S) { /* Does the counter reach 24 hours? */
    TotalHthH -= 24*3600UL*TmDt1_TICKS_PER_S; /* If yes then reset it by subtracting exactly 24 hours */
    CntDay++;                          /* Increment day counter */
    if (CntYear & 0x03U) {             /* Is this year un-leap-one? */
      ptr = ULY;                       /* Set pointer to un-leap-year day table */
    } else {                           /* Is this year leap-one? */
      ptr = LY;                        /* Set pointer to leap-year day table */
    }
    ptr--;                             /* Decrement the pointer */
    if (CntDay > ptr[CntMonth]) {      /* Day counter overflow? */
      CntDay = 1U;                     /* Set day counter on 1 */
      CntMonth++;                      /* Increment month counter */
      if (CntMonth > 12U) {            /* Month counter overflow? */
        CntMonth = 1U;                 /* Set month counter on 1 */
        CntYear++;                     /* Increment year counter */
      }
    }
  }
}

/*
** ===================================================================
**     Method      :  TmDt1_GetTime (component GenericTimeDate)
**     Description :
**         This method returns current time.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Time            - Pointer to the structure TIMEREC. It
**                           contains actual number of hours, minutes,
**                           seconds and hundredth of seconds.
**     Returns     :
**         ---             - Error code
** ===================================================================
*/
byte TmDt1_GetTime(TIMEREC *Time)
{
  dword Var1;                          /* Working temporary copy of software tick counter */
  word Var2;                           /* Working temporary variable */

  EnterCritical();                     /* Save the PS register */
  Var1 = TotalHthH;                    /* Loading actual number of tens of ms */
  ExitCritical();                      /* Restore the PS register */
  Time->Sec100 = (byte)(Var1 % 100U);  /* Modulo 100 gives appropriate number of hundreds of seconds */
  Var1 = (dword)(Var1 / 100UL);        /* Quotient gives total number of seconds then */
  Time->Sec = (byte)(Var1 % 60U);      /* Modulo 60 gives appropriate number of seconds */
  Var2 = (word)(Var1 / 60UL);          /* Quotient gives total number of minutes then */
  Time->Min = (byte)(Var2 % 60U);      /* Modulo 60 gives appropriate number of minutes */
  Time->Hour = (byte)(Var2 / 60U);     /* Quotient gives total number of hours then */
  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  TmDt1_SetDate (component GenericTimeDate)
**     Description :
**         This method sets a new actual date.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Year            - Years (16-bit unsigned integer)
**         Month           - Months (8-bit unsigned integer)
**         Day             - Days (8-bit unsigned integer)
**     Returns     :
**         ---             - Error code
** ===================================================================
*/
byte TmDt1_SetDate(word Year, byte Month, byte Day)
{
  const byte * ptr;                    /* Pointer to ULY/LY table */

  if ((Year < 1998U) || (Year > 2099U) || (Month > 12U) || (Month == 0U) || (Day > 31U) || (Day == 0U)) { /* Test correctness of given parameters */
    return ERR_RANGE;                  /* If not correct then error */
  }
  ptr = (((Year & 0x03U) != 0U) ? ULY : LY); /* Set pointer to leap-year or un-leap-year day table */
  if (ptr[Month - 1U] < Day) {         /* Does the obtained number of days exceed number of days in the appropriate month & year? */
    return ERR_RANGE;                  /* If yes (incorrect date inserted) then error */
  }
  EnterCritical();                     /* Save the PS register */
  CntDay = Day;                        /* Set day counter to the given value */
  CntMonth = Month;                    /* Set month counter to the given value */
  CntYear = Year;                      /* Set year counter to the given value */
  ExitCritical();                      /* Restore the PS register */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TmDt1_GetDate (component GenericTimeDate)
**     Description :
**         This method returns current date.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Date            - Pointer to DATEREC
**     Returns     :
**         ---             - Error code
** ===================================================================
*/
byte TmDt1_GetDate(DATEREC *Date)
{
  EnterCritical();                     /* Save the PS register */
  Date->Year = CntYear;                /* Year */
  Date->Month = CntMonth;              /* Month */
  Date->Day = CntDay;                  /* Day */
  ExitCritical();                      /* Restore the PS register */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TmDt1_Init (component GenericTimeDate)
**     Description :
**         Initialization method
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void TmDt1_Init(void)
{
  /* initialize date/time as set in properties */
  (void)TmDt1_SetTime(17, 51, 31, 0);
  (void)TmDt1_SetDate(2012, 7, 22);
}

/* END TmDt1. */

/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.2 [05.06]
**     for the Freescale Kinetis series of microcontrollers.
**
** ###################################################################
*/
